package info.msxlaunchers.openmsx.game.scan;

import info.msxlaunchers.openmsx.game.repository.RepositoryData;
import info.msxlaunchers.openmsx.game.scan.FileScanner;
import info.msxlaunchers.openmsx.launcher.builder.GameBuilder;
import info.msxlaunchers.openmsx.launcher.data.game.Game;
import info.msxlaunchers.openmsx.launcher.extra.ExtraDataGetter;
import info.msxlaunchers.openmsx.launcher.persistence.game.DatabaseAlreadyExistsException;
import info.msxlaunchers.openmsx.launcher.persistence.game.DatabaseMaxBackupReachedException;
import info.msxlaunchers.openmsx.launcher.persistence.game.DatabaseNotFoundException;
import info.msxlaunchers.openmsx.launcher.persistence.game.DatabaseNullNameException;
import info.msxlaunchers.openmsx.launcher.persistence.game.GamePersister;
import info.msxlaunchers.openmsx.launcher.persistence.game.GameWithNullNameException;

import java.io.IOException;

import org.junit.Test;
import org.junit.runner.RunWith;

import static org.junit.Assert.assertEquals;

import org.mockito.Mock;
import org.mockito.runners.MockitoJUnitRunner;

import static org.mockito.Matchers.anyLong;
import static org.mockito.Matchers.anyMap;
import static org.mockito.Matchers.anyString;
import static org.mockito.Mockito.doThrow;
import static org.mockito.Mockito.never;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

@RunWith( MockitoJUnitRunner.class )
public class FileScannerTest
{
	private final String database = "testDatabase";
	private final String path1 = getClass().getResource( "dir1" ).getFile();
	private final String path2 = getClass().getResource( "dir2" ).getFile();
	private final String[] paths = new String[] { path1, path2 };

	@Mock GamePersister gamePersister;
	@Mock RepositoryData repositoryData;
	@Mock GameBuilder gameBuilder;
	@Mock ExtraDataGetter extraDataGetter;

	@Test( expected = NullPointerException.class )
	public void testForNullConditionsConstructorArg1() throws IOException
	{
		new FileScanner( null, repositoryData, gameBuilder, extraDataGetter );
	}

	@Test( expected = NullPointerException.class )
	public void testForNullConditionsConstructorArg2() throws IOException
	{
		new FileScanner( gamePersister, null, gameBuilder, extraDataGetter );
	}

	@Test( expected = NullPointerException.class )
	public void testForNullConditionsConstructorArg3() throws IOException
	{
		new FileScanner( gamePersister, repositoryData, null, extraDataGetter );
	}

	@Test( expected = NullPointerException.class )
	public void testForNullConditionsConstructorArg4() throws IOException
	{
		new FileScanner( gamePersister, repositoryData, gameBuilder, null );
	}

	@Test( expected = DatabaseNotFoundException.class )
	public void testForBackupNonExistentDatabase()
			throws GameWithNullNameException, DatabaseMaxBackupReachedException, DatabaseNullNameException, DatabaseAlreadyExistsException, DatabaseNotFoundException, IOException
	{
		FileScanner scanner = new FileScanner( gamePersister, repositoryData, gameBuilder, extraDataGetter );

		doThrow( new DatabaseNotFoundException( database ) ).when( gamePersister ).backupDatabase( database );

		//set backup to true
		//newDatabase must be false
		scanner.scan( paths, true, database, false, true, "machine", true, true, true, true, false, true );
	}

	@Test( expected = DatabaseNotFoundException.class )
	public void testForOverwriteNonExistentDatabase()
			throws GameWithNullNameException, DatabaseMaxBackupReachedException, DatabaseNullNameException, DatabaseAlreadyExistsException, DatabaseNotFoundException, IOException
	{
		FileScanner scanner = new FileScanner( gamePersister, repositoryData, gameBuilder, extraDataGetter );

		doThrow( new DatabaseNotFoundException( database ) ).when( gamePersister ).recreateDatabase( database );

		//set append to false
		//newDatabase must be false
		scanner.scan( paths, true, database, false, false, "machine", true, true, true, true, false, false );
	}

	@Test
	public void testNewDatabaseOptionOn()
			throws GameWithNullNameException, DatabaseMaxBackupReachedException, DatabaseNullNameException, DatabaseAlreadyExistsException, DatabaseNotFoundException, IOException
	{
		FileScanner scanner = new FileScanner( gamePersister, repositoryData, gameBuilder, extraDataGetter );

		//set newDatabase to true
		//setting the append flag to true is irrelevant in this case
		scanner.scan( paths, true, database, true, true, "machine", true, true, true, true, false, true );

		//check that the createDatabase was called
		verify( gamePersister ).createDatabase( database );
	}

	@Test
	public void testNewDatabaseOptionOff()
			throws GameWithNullNameException, DatabaseMaxBackupReachedException, DatabaseNullNameException, DatabaseAlreadyExistsException, DatabaseNotFoundException, IOException
	{
		FileScanner scanner = new FileScanner( gamePersister, repositoryData, gameBuilder, extraDataGetter );

		//set newDatabase to false
		//setting the append flag to true is irrelevant in this case
		scanner.scan( paths, true, database, false, true, "machine", true, true, true, true, false, true );

		//check that the createDatabase was called
		verify( gamePersister, never() ).createDatabase( database );
	}

	@Test
	public void testBackupDatabaseOptionOn()
			throws GameWithNullNameException, DatabaseMaxBackupReachedException, DatabaseNullNameException, DatabaseAlreadyExistsException, DatabaseNotFoundException, IOException
	{
		FileScanner scanner = new FileScanner( gamePersister, repositoryData, gameBuilder, extraDataGetter );

		//set backup to true
		//newDatabase must be false
		scanner.scan( paths, true, database, false, true, "machine", true, true, true, true, false, true );

		//check that the backupDatabase was called
		verify( gamePersister ).backupDatabase( database );
	}

	@Test
	public void testBackupDatabaseOptionOff()
			throws GameWithNullNameException, DatabaseMaxBackupReachedException, DatabaseNullNameException, DatabaseAlreadyExistsException, DatabaseNotFoundException, IOException
	{
		FileScanner scanner = new FileScanner( gamePersister, repositoryData, gameBuilder, extraDataGetter );

		//set backup to false
		//newDatabase must be false
		scanner.scan( paths, true, database, false, true, "machine", true, true, true, true, false, false );

		//check that the backupDatabase was not called
		verify( gamePersister, never() ).backupDatabase( database );
	}

	@Test
	public void testAppendToDatabaseOptionOn()
			throws GameWithNullNameException, DatabaseMaxBackupReachedException, DatabaseNullNameException, DatabaseAlreadyExistsException, DatabaseNotFoundException, IOException
	{
		FileScanner scanner = new FileScanner( gamePersister, repositoryData, gameBuilder, extraDataGetter );

		//set append to true
		//newDatabase must be false
		scanner.scan( paths, true, database, false, true, "machine", true, true, true, true, false, true );

		//check that the recreateDatabase was not called
		verify( gamePersister, never() ).recreateDatabase( database );
	}

	@Test
	public void testAppendToDatabaseOptionOff()
			throws GameWithNullNameException, DatabaseMaxBackupReachedException, DatabaseNullNameException, DatabaseAlreadyExistsException, DatabaseNotFoundException, IOException
	{
		FileScanner scanner = new FileScanner( gamePersister, repositoryData, gameBuilder, extraDataGetter );

		//set append to false
		//newDatabase must be false
		scanner.scan( paths, true, database, false, false, "machine", true, true, true, true, false, false );

		//check that the recreateDatabase was called
		verify( gamePersister ).recreateDatabase( database );
	}

	@Test
	public void testGetNameFromOpenMSXDatabaseOptionOn()
			throws GameWithNullNameException, DatabaseMaxBackupReachedException, DatabaseNullNameException, DatabaseAlreadyExistsException, DatabaseNotFoundException, IOException
	{
		FileScanner scanner = new FileScanner( gamePersister, repositoryData, gameBuilder, extraDataGetter );

		//set getNameFromOpenMSXDatabase to true
		scanner.scan( paths, true, database, false, true, "machine", true, true, true, true, true, true );

		//check that the getRepositoryInfo was called
		verify( repositoryData ).getRepositoryInfo();
	}

	@Test
	public void testGetNameFromOpenMSXDatabaseOptionOff()
			throws GameWithNullNameException, DatabaseMaxBackupReachedException, DatabaseNullNameException, DatabaseAlreadyExistsException, DatabaseNotFoundException, IOException
	{
		FileScanner scanner = new FileScanner( gamePersister, repositoryData, gameBuilder, extraDataGetter );

		//set getNameFromOpenMSXDatabase to false
		scanner.scan( paths, true, database, false, false, "machine", true, true, true, true, false, false );

		//check that the getRepositoryInfo was not called
		verify( repositoryData, never() ).getRepositoryInfo();
	}

	@Test @SuppressWarnings("unchecked")
	public void testSearchForRomsOnly()
			throws GameWithNullNameException, DatabaseMaxBackupReachedException, DatabaseNullNameException, DatabaseAlreadyExistsException, DatabaseNotFoundException, IOException
	{
		FileScanner scanner = new FileScanner( gamePersister, repositoryData, gameBuilder, extraDataGetter );

		when( gameBuilder
				.createGameObjectForScannedFiles( anyString(), anyString(), anyString(), anyString(), anyString(), anyString(), anyString(), anyString(), anyString(), anyLong(), anyMap() ) )
				.thenReturn( Game.name("name").build() );

		//traverse all test directories
		int found = scanner.scan( paths, true, database, false, false, "machine", true, false, false, false, false, false );

		//there are 4 Roms in the test directories
		assertEquals( 4, found );
	}

	@Test @SuppressWarnings("unchecked")
	public void testSearchForDisksOnly()
			throws GameWithNullNameException, DatabaseMaxBackupReachedException, DatabaseNullNameException, DatabaseAlreadyExistsException, DatabaseNotFoundException, IOException
	{
		FileScanner scanner = new FileScanner( gamePersister, repositoryData, gameBuilder, extraDataGetter );

		when( gameBuilder
				.createGameObjectForScannedFiles( anyString(), anyString(), anyString(), anyString(), anyString(), anyString(), anyString(), anyString(), anyString(), anyLong(), anyMap() ) )
				.thenReturn( Game.name("name").build());

		//traverse all test directories
		int found = scanner.scan( paths, true, database, false, false, "machine", false, true, false, false, false, false );

		//there are 4 disks in the test directories
		assertEquals( 4, found );
	}

	@Test @SuppressWarnings("unchecked")
	public void testSearchForTapesOnly()
			throws GameWithNullNameException, DatabaseMaxBackupReachedException, DatabaseNullNameException, DatabaseAlreadyExistsException, DatabaseNotFoundException, IOException
	{
		FileScanner scanner = new FileScanner( gamePersister, repositoryData, gameBuilder, extraDataGetter );

		when( gameBuilder
				.createGameObjectForScannedFiles( anyString(), anyString(), anyString(), anyString(), anyString(), anyString(), anyString(), anyString(), anyString(), anyLong(), anyMap() ) )
				.thenReturn( Game.name("name").build());

		//traverse all test directories
		int found = scanner.scan( paths, true, database, false, false, "machine", false, false, true, false, false, false );

		//there are 3 tapes in the test directories
		assertEquals( 3, found );
	}

	@Test @SuppressWarnings("unchecked")
	public void testSearchForLaserdiscsOnly()
			throws GameWithNullNameException, DatabaseMaxBackupReachedException, DatabaseNullNameException, DatabaseAlreadyExistsException, DatabaseNotFoundException, IOException
	{
		FileScanner scanner = new FileScanner( gamePersister, repositoryData, gameBuilder, extraDataGetter );

		when( gameBuilder
				.createGameObjectForScannedFiles( anyString(), anyString(), anyString(), anyString(), anyString(), anyString(), anyString(), anyString(), anyString(), anyLong(), anyMap() ) )
				.thenReturn( Game.name("name").build());

		//traverse all test directories
		int found = scanner.scan( paths, true, database, false, false, "machine", false, false, false, true, false, false );

		//there is 3 laserdiscs in the test directories
		assertEquals( 3, found );
	}

	@Test @SuppressWarnings("unchecked")
	public void testSearchForAllMediaWithTraverseOptionOn()
			throws GameWithNullNameException, DatabaseMaxBackupReachedException, DatabaseNullNameException, DatabaseAlreadyExistsException, DatabaseNotFoundException, IOException
	{
		FileScanner scanner = new FileScanner( gamePersister, repositoryData, gameBuilder, extraDataGetter );

		when( gameBuilder
				.createGameObjectForScannedFiles( anyString(), anyString(), anyString(), anyString(), anyString(), anyString(), anyString(), anyString(), anyString(), anyLong(), anyMap() ) )
				.thenReturn( Game.name("name").build());

		//traverse all test directories
		int found = scanner.scan( paths, true, database, false, false, "machine", true, true, true, true, false, false );

		//there are 14 total in the test directories
		assertEquals( 14, found );
	}

	@Test @SuppressWarnings("unchecked")
	public void testSearchForAllMediaWithTraverseOptionOff()
			throws GameWithNullNameException, DatabaseMaxBackupReachedException, DatabaseNullNameException, DatabaseAlreadyExistsException, DatabaseNotFoundException, IOException
	{
		FileScanner scanner = new FileScanner( gamePersister, repositoryData, gameBuilder, extraDataGetter );

		when( gameBuilder
				.createGameObjectForScannedFiles( anyString(), anyString(), anyString(), anyString(), anyString(), anyString(), anyString(), anyString(), anyString(), anyLong(), anyMap() ) )
				.thenReturn( Game.name("name").build());

		//don't traverse
		int found = scanner.scan( paths, false, database, false, false, "machine", true, true, true, true, false, false );

		//there are 10 total in the test directories
		assertEquals( 10, found );
	}
}
