package info.msxlaunchers.openmsx.launcher.ui.presenter;

import info.msxlaunchers.openmsx.launcher.data.settings.constants.Language;
import info.msxlaunchers.openmsx.launcher.persistence.game.DatabaseNotFoundException;
import info.msxlaunchers.openmsx.launcher.persistence.game.GamePersister;
import info.msxlaunchers.openmsx.launcher.ui.view.DatabaseManagerView;

import java.io.IOException;
import java.util.Set;

import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;

import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.runners.MockitoJUnitRunner;

import static org.mockito.Matchers.any;
import static org.mockito.Matchers.anyBoolean;
import static org.mockito.Mockito.eq;
import static org.mockito.Mockito.times;
import static org.mockito.Mockito.verify;

@RunWith( MockitoJUnitRunner.class )
public class DatabaseManagerPresenterImplTest
{
	@Mock DatabaseManagerView view;
	@Mock MainPresenter mainPresenter;
	@Mock GamePersister gamePersister;
	@Mock Set<String> databases;

	private DatabaseManagerPresenterImpl presenter;
	private String database = "database";

	@Before
	public void setUp() throws IOException
	{
		presenter = new DatabaseManagerPresenterImpl( view, mainPresenter, gamePersister );
	}

	@Test
	public void testConstructor() throws IOException
	{
		new DatabaseManagerPresenterImpl( view, mainPresenter, gamePersister );
	}

	@Test( expected = NullPointerException.class )
	public void testConstructorArg1Null() throws IOException
	{
		new DatabaseManagerPresenterImpl( null, mainPresenter, gamePersister );
	}

	@Test( expected = NullPointerException.class )
	public void testConstructorArg2Null() throws IOException
	{
		new DatabaseManagerPresenterImpl( view, null, gamePersister );
	}

	@Test( expected = NullPointerException.class )
	public void testConstructorArg3Null() throws IOException
	{
		new DatabaseManagerPresenterImpl( view, mainPresenter, null );
	}

	@Test
	public void test_WhenCallOnRequestDatabaseManagerScreen_ThenViewDisplayScreenIsCalled()
	{
		presenter.onRequestDatabaseManagerScreen( databases, Language.DUTCH, false );

		verify( view, times( 1 ) ).displayScreen( Language.DUTCH, false, databases );
	}

	@Test
	public void test_WhenCallOnRequestDeleteDatabase_ThenGamePersisterDeleteDatabaseIsCalled() throws LauncherException, DatabaseNotFoundException, IOException
	{
		presenter.onRequestDeleteDatabase( database );

		verify( gamePersister, times( 1 ) ).deleteDatabase( database );
		verify( mainPresenter, times( 1 ) ).onRequestDeleteDatabaseAction( database );
	}

	@Test( expected = LauncherException.class )
	public void test_GivenNonExistingDatabase_WhenCallOnRequestDeleteDatabase_ThenExceptionIsThrown() throws DatabaseNotFoundException, IOException, LauncherException
	{
		Mockito.doThrow( new DatabaseNotFoundException( database ) ).when( gamePersister ).deleteDatabase( database );

		presenter.onRequestDeleteDatabase( database );
	}

	@Test( expected = LauncherException.class )
	public void test_GivenIOException_WhenCallOnRequestDeleteDatabase_ThenExceptionIsThrown() throws DatabaseNotFoundException, IOException, LauncherException
	{
		Mockito.doThrow( new IOException() ).when( gamePersister ).deleteDatabase( database );

		presenter.onRequestDeleteDatabase( database );
	}

	@Test
	public void test_WhenCallOnRequestRenameDatabaseScreen_ThenViewDisplayScreenIsCalled() throws LauncherException
	{
		presenter.onRequestRenameDatabaseScreen( database );

		verify( view, times( 1 ) ).displayRenameDatabaseScreen( eq( database ), any( Language.class ), anyBoolean() );
	}

	@Test
	public void test_WhenCallOnRequestRenameDatabaseAction_ThenGamePersisterRenameDatabaseIsCalled() throws LauncherException, DatabaseNotFoundException, IOException
	{
		String newDatabase = "newDatabase";

		presenter.onRequestRenameDatabaseAction( database, newDatabase );

		verify( gamePersister, times( 1 ) ).renameDatabase( database, newDatabase );
		verify( mainPresenter, times( 1 ) ).onRequestRenameDatabaseAction( database, newDatabase );
	}

	@Test( expected = LauncherException.class )
	public void test_GivenNonExistingDatabase_WhenCallOnRequestRenameDatabaseAction_ThenExceptionIsThrown() throws DatabaseNotFoundException, IOException, LauncherException
	{
		String newDatabase = "newDatabase";
		
		Mockito.doThrow( new DatabaseNotFoundException( database ) ).when( gamePersister ).renameDatabase( database, newDatabase );

		presenter.onRequestRenameDatabaseAction( database, newDatabase );
	}

	@Test( expected = LauncherException.class )
	public void test_GivenIOException_WhenCallOnRequestRenameDatabaseAction_ThenExceptionIsThrown() throws DatabaseNotFoundException, IOException, LauncherException
	{
		String newDatabase = "newDatabase";
		
		Mockito.doThrow( new IOException() ).when( gamePersister ).renameDatabase( database, newDatabase );

		presenter.onRequestRenameDatabaseAction( database, newDatabase );
	}
}